#ifndef NPSTAT_LAPACK_H_
#define NPSTAT_LAPACK_H_

#ifdef __cplusplus
extern "C" {
#endif

/* SVD solver for a linear least squares problem which minimizes
 * L2 norm of (A*x - b)
 */
void dgelsd_(int *M, int *N, int *NRHS, double *A,
             int *LDA, double *B, int *LDB, double *S,
             double *RCOND, int *RANK, double *WORK,
             int *LWORK, int *IWORK, int *INFO);

/* Solve a system of linear equations */
void dgetrs_(char *TRANS, int *N, int *NRHS, double *A, int *LDA, int *IPIV,
	     double *B, int *LDB, int *INFO, int lenTRANS);
void sgetrs_(char *TRANS, int *N, int *NRHS, float *A, int *LDA, int *IPIV,
	     float *B, int *LDB, int *INFO, int lenTRANS);

/* Calculation of eigenvalues and (optionally) eigenvectors
 * of a symmetric matrix
 */
void dsyev_(char *JOBZ, char *UPLO, int *N, double *A, int *LDA, double *W,
            double *WORK, int *LWORK, int *INFO, int lenJOBZ, int lenUPLO);
void ssyev_(char *JOBZ, char *UPLO, int *N, float *A, int *LDA, float *W,
            float *WORK, int *LWORK, int *INFO, int lenJOBZ, int lenUPLO);

/* Calculation of eigenvalues and (optionally) eigenvectors
 * of a symmetric matrix using divide and conquer algorithm
 */
void dsyevd_(char *JOBZ, char *UPLO, int *N, double *A, int *LDA, double *W,
             double *WORK, int *LWORK, int *IWORK, int *LIWORK,
             int *INFO, int lenJOBZ, int lenUPLO);
void ssyevd_(char *JOBZ, char *UPLO, int *N, float *A, int *LDA, float *W,
             float *WORK, int *LWORK, int *IWORK, int *LIWORK,
             int *INFO, int lenJOBZ, int lenUPLO);

/* Calculation of eigenvalues and (optionally) eigenvectors
 * of a symmetric matrix using "Relatively Robust Representations"
 */
void dsyevr_(char *JOBZ, char *RANGE, char *UPLO,
             int *N, double *A, int *LDA, double *VL, double *VU,
             int *IL, int *IU, double *ABSTOL, int *M,
             double *W, double *Z, int *LDZ, int *ISUPPZ,
             double *WORK, int *LWORK, int *IWORK, int *LIWORK,
             int *INFO, int lenJOBZ, int lenRANGE, int lenUPLO);
void ssyevr_(char *JOBZ, char *RANGE, char *UPLO,
             int *N, float *A, int *LDA, float *VL, float *VU,
             int *IL, int *IU, float *ABSTOL, int *M,
             float *W, float *Z, int *LDZ, int *ISUPPZ,
             float *WORK, int *LWORK, int *IWORK, int *LIWORK,
             int *INFO, int lenJOBZ, int lenRANGE, int lenUPLO);

/* LU factorization */
void dgetrf_(int *M, int *N, double *A, int *LDA, int *IPIV, int *INFO);
void sgetrf_(int *M, int *N, float *A, int *LDA, int *IPIV, int *INFO);

/* Cholesky factorization */
void dpotrf_(char *UPLO, int *N, double *A, int *LDA, int *INFO, int lenUPLO);
void spotrf_(char *UPLO, int *N, float *A, int *LDA, int *INFO, int lenUPLO);

/* Factorization of a real symmetric matrix using Bunch-Kaufman
 * diagonal pivoting method
 */
void dsytrf_(char *UPLO, int *N, double *A, int *LDA, int *IPIV,
             double *WORK, int *LWORK, int *INFO, int lenUPLO);
void ssytrf_(char *UPLO, int *N, float *A, int *LDA, int *IPIV,
             float *WORK, int *LWORK, int *INFO, int lenUPLO);

/* Inverse of a general matrix */
void dgetri_(int *N, double *A, int *LDA, int *IPIV,
             double *WORK, int *LWORK, int *INFO);
void sgetri_(int *N, float *A, int *LDA, int *IPIV,
             float *WORK, int *LWORK, int *INFO);

/* Inverse of a symmetric matrix */
void dsytri_(char *UPLO, int *N, double *A, int *LDA, int *IPIV,
             double *WORK, int *INFO, int lenUPLO);
void ssytri_(char *UPLO, int *N, float *A, int *LDA, int *IPIV,
             float *WORK, int *INFO, int lenUPLO);

/* Inverse of a symmetric positive definite matrix */
void dpotri_(char *UPLO, int *N, double *A, int *LDA, int *INFO, int lenUPLO);
void spotri_(char *UPLO, int *N, float *A, int *LDA, int *INFO, int lenUPLO);

#ifdef __cplusplus
}
#endif

#endif // NPSTAT_LAPACK_H_
