#ifndef NPSTAT_ABSCOMPOSITEDISTROBUILDER_HH_
#define NPSTAT_ABSCOMPOSITEDISTROBUILDER_HH_

/*!
// \file AbsCompositeDistroBuilder.hh
//
// \brief Interface definition for classes which build composite distrubutions
//
// Author: I. Volobouev
//
// September 2010
*/

#include <vector>

#include "npstat/stat/OrderedPointND.hh"
#include "npstat/stat/CompositeDistributionND.hh"

namespace npstat {
    /**
    // Interface definition for classes which build composite distrubutions
    // out of data samples (typically, by fitting or smoothing) -- see header
    // CompositeDistributionND.hh for more details on these distributions.
    */
    template <class Point>
    class AbsCompositeDistroBuilder
    {
    public:
        virtual ~AbsCompositeDistroBuilder() {}

        /**
        // This method will build CompositeDistributionND
        // object on the heap. It is a responsibility of
        // the user to delete it when it is no longer needed.
        */
        CompositeDistributionND* build(
            std::vector<OrderedPointND<Point> >& data) const;

    private:
        // The following methods should construct the relevant
        // objects on the heap and must be implemented by the
        // derived classes
        virtual AbsDistribution1D* buildMarginal(
            unsigned dimNumber,
            const std::vector<typename Point::value_type>&) const = 0;

        virtual AbsDistributionND* buildCopula(
            std::vector<OrderedPointND<Point> >&) const = 0;
    };
}

#include "npstat/stat/AbsCompositeDistroBuilder.icc"

#endif // NPSTAT_ABSCOMPOSITEDISTROBUILDER_HH_
