#include <cassert>
#include <stdexcept>

#include "npstat/stat/AbsCopulaSmoother.hh"

namespace npstat{
    static std::vector<npstat::HistoAxis> buildCopulaAxes(
        const unsigned* nBinsInEachDim, const unsigned dim)
    {
        if (dim <= 1U) throw std::invalid_argument(
            "In npstat::buildCopulaAxes: copula dimensionality "
            "must be at least 2");
        assert(nBinsInEachDim);
        std::vector<npstat::HistoAxis> axes;
        axes.reserve(dim);
        for (unsigned i=0; i<dim; ++i)
            axes.push_back(npstat::HistoAxis(nBinsInEachDim[i], 0.0, 1.0));
        return axes;
    }

    AbsCopulaSmoother::AbsCopulaSmoother(
        const unsigned* nBinsInEachDim, const unsigned dim,
        const double tolerance, const unsigned nNormalizationCycles)
        : h_(buildCopulaAxes(nBinsInEachDim, dim)),
          tol_(tolerance), nCycles_(nNormalizationCycles)
    {
    }
}
