#ifndef NPSTAT_ABSFILTER1DBUILDER_HH_
#define NPSTAT_ABSFILTER1DBUILDER_HH_

/*!
// \file AbsFilter1DBuilder.hh
//
// \brief Abstract interface for building local polynomial filter weights in 1-d
//
// Author: I. Volobouev
//
// November 2009
*/

#include <vector>

namespace npstat {
    /** 
    // Besides providing a table of weights, the filter should remember
    // for which point it was constructed (this info can potentially be
    // used in subsequent cross-validation calculations)
    */
    class PolyFilter1D : public std::vector<long double>
    {
    public:
        inline explicit PolyFilter1D(const unsigned peak) : peak_(peak) {}

        inline unsigned peakPosition() const {return peak_;}

    private:
        PolyFilter1D();
        unsigned peak_;
    };

    /** 
    // Abstract interface class for building local polynomial filter
    // weights in 1-d
    */
    struct AbsFilter1DBuilder
    {
        inline virtual ~AbsFilter1DBuilder() {}

        /**
        // Length of the filter constructed at a point deeply inside
        // the density support region
        */
        virtual unsigned centralWeightLength() const = 0;

        /**
        // Build the filter from the given taper function, maximum
        // polynomial degree, bin number for which this filter is
        // constructed, and expected length of the data
        */
        virtual PolyFilter1D* makeFilter(const double* taper,
                                         unsigned maxDegree,
                                         unsigned binnum,
                                         unsigned datalen) const = 0;
    };
}

#endif // NPSTAT_ABSFILTER1DBUILDER_HH_
