#ifndef NPSTAT_BANDWIDTHCVPSEUDOLOGLI1D_HH_
#define NPSTAT_BANDWIDTHCVPSEUDOLOGLI1D_HH_

/*!
// \file BandwidthCVPseudoLogli1D.hh
//
// \brief Cross-validating one-dimensional density estimates by optimizing
//        regularized pseudo log-likelihood
//
// Author: I. Volobouev
//
// September 2010
*/

#include "npstat/stat/AbsBandwidthCV.hh"

namespace npstat {
    /**
    // Class for calculating KDE or LOrPE cross-validation pseudo
    // log-likelihood, for 1-dimensional density estimates
    */
    template<typename Numeric, typename Num2>
    class BandwidthCVPseudoLogli1D : public AbsBandwidthCV1D<Numeric,Num2>
    {
    public:
        /**
        // Parameter "regularizationPower" is used to limit the contributions
        // into the overall pseudo log likelihood from points for which the
        // "leaving one out" density is very low. For those points, instead
        // of "leaving one out" density, we will use the density generated by
        // that point itself divided by pow(N, regularizationPower). This
        // method limits the effect of tails on bandwidth determination.
        */
        inline BandwidthCVPseudoLogli1D(double regularizationPower=0.5)
            : renormPow_(regularizationPower) {}
        inline virtual ~BandwidthCVPseudoLogli1D() {}

        double operator()(
            const HistoND<Numeric>& histo,
            const Num2* densityEstimate,
            unsigned lenEstimate,
            const AbsPolyFilter1D& filterUsed) const;

    private:
        double renormPow_;
    };
}

#include "npstat/stat/BandwidthCVPseudoLogli1D.icc"

#endif // NPSTAT_BANDWIDTHCVPSEUDOLOGLI1D_HH_
