#include <stdexcept>

#include "npstat/stat/LOrPEMarginalSmoother.hh"
#include "npstat/stat/lorpeSmooth1D.hh"

namespace npstat {
    LOrPEMarginalSmoother::LOrPEMarginalSmoother(
        const unsigned nbins, const double xmin, const double xmax,
        const int isymbetaPower, const unsigned ipolyDegree,
        const double ibwFactor, const bool stretch)
        : AbsMarginalSmoother(nbins, xmin, xmax),
          bwFactor_(ibwFactor),
          symbetaPower_(isymbetaPower),
          polyDegree_(ipolyDegree),
          stretchFilter_(stretch)
    {
        if (bwFactor_ <= 0.0) throw std::invalid_argument(
            "In npstat::LOrPEMarginalSmoother constructor: "
            "bandwidth factor must be positive");
    }

    void LOrPEMarginalSmoother::smoothHistogram(HistoND<double>& histo,
                                                double* bandwidthUsed)
    {
        const double bandwidth = lorpeSmooth1D(
            histo, symbetaPower_, polyDegree_,
            const_cast<double*>(histo.binContents().data()),
            histo.nBins(), bwFactor_, stretchFilter_);
        if (bandwidthUsed)
            *bandwidthUsed = bandwidth;
    }

    double LOrPEMarginalSmoother::selfContribution(unsigned) const
    {
        throw std::runtime_error(
            "In npstat::LOrPEMarginalSmoother::selfContribution: "
            "this function is not implemented yet");
        return 0.0;
    }
}
