#ifndef NPSTAT_LORPEMARGINALSMOOTHER_HH_
#define NPSTAT_LORPEMARGINALSMOOTHER_HH_

#include "npstat/stat/AbsMarginalSmoother.hh"

/*!
// \file LOrPEMarginalSmoother.hh
//
// \brief LOrPE adapter for automatic smoothing of 1-d marginals
//
// Author: I. Volobouev
//
// March 2013
*/

namespace npstat
{
    class LOrPEMarginalSmoother : public AbsMarginalSmoother
    {
    public:
        LOrPEMarginalSmoother(unsigned nbins, double xmin, double xmax,
                              int symbetaPower, unsigned polyDegree,
                              double bandwidthfactor,
                              bool stretchFilterAtTheBoundary=false);

        virtual ~LOrPEMarginalSmoother() {}

        //@{
        /** Inspect object properties */
        inline int symbetaPower() const {return symbetaPower_;}
        inline double bwFactor() const {return bwFactor_;}
        inline unsigned polyDegree() const {return polyDegree_;}
        inline bool stretchFilterAtTheBoundary() const {return stretchFilter_;}
        //@}

        /**
        // This method is not implemented yet. Will throw
        // std::runtime_error if called.
        */
        double selfContribution(unsigned binNumber) const;

    private:
        virtual void smoothHistogram(HistoND<double>& histo,
                                     double* bandwidthUsed);
        double bwFactor_;
        int symbetaPower_; 
        unsigned polyDegree_;
        bool stretchFilter_;
    };
}

#endif // NPSTAT_LORPEMARGINALSMOOTHER_HH_
