#include "npstat/stat/SequentialCopulaSmoother.hh"
#include "npstat/stat/Filter1DBuilders.hh"

namespace npstat {
    SequentialCopulaSmoother::SequentialCopulaSmoother(
        const unsigned* nBinsInEachDim, const unsigned dim,
        const double marginTolerance, const unsigned nNormCycles,
        const AbsDistribution1D& kernel,
        const double* taper, const unsigned maxDegree,
        const double initialBw, const double* bwCoeffs,
        const CVCalc* cvCalc, const bool assumeCvCalcOwnership,
        double cvFactor, const unsigned nCV, const bool useConvolute)
        : CVCopulaSmoother<SequentialPolyFilterND>(
            nBinsInEachDim, dim, marginTolerance, nNormCycles, initialBw,
            cvCalc, assumeCvCalcOwnership, cvFactor, nCV, useConvolute)
    {
        const std::vector<double>& bwVec = bandwidthValues();
        const unsigned nFilters = bwVec.size();
        if (nFilters)
        {
            // Buffer for scanned kernel values
            unsigned nmax = nBinsInEachDim[0];
            for (unsigned idim=1; idim<dim; ++idim)
                if (nBinsInEachDim[idim] > nmax)
                    nmax = nBinsInEachDim[idim];
            std::vector<double> kernelBuf(nmax);
            double* w = &kernelBuf[0];

            // Generate all filters
            std::vector<const LocalPolyFilter1D*> f1d(dim, 0);
            for (unsigned ifilt=0; ifilt<nFilters; ++ifilt)
            {
                for (unsigned idim=0; idim<dim; ++idim)
                {
                    const unsigned nbins = nBinsInEachDim[idim];
                    const double binwidth = 1.0/nbins;
                    const double bandwidth = bwCoeffs ? 
                        bwVec[ifilt]*bwCoeffs[idim] : bwVec[ifilt];

                    unsigned imax = 0;
                    for (; imax<nbins; ++imax)
                    {
                        const double x = binwidth*imax/bandwidth;
                        w[imax] = kernel.density(x)/bandwidth;
                        if (w[imax] <= 0.0)
                            break;
                    }

                    WeightTableFilter1DBuilder builder(w, imax);
                    f1d[idim] = new LocalPolyFilter1D(taper, maxDegree,
                                                      builder, nbins);
                }

                // Generate the filter
                Filter* f = new SequentialPolyFilterND(&f1d[0], dim, true);
                this->setFilter(ifilt, f);
            }
        }
    }
}
